@@ -1,6 +1,6 @@
 from web3 import Web3
 from ens import ENS
-from actions_lib.utils.contact_tool import redis_add_contact, redis_show_contact
+from actions_lib.utils.contact_tool import redis_add_contact, redis_edit_contact, redis_delete_contact, redis_show_contact
 import logging
 
 logger = logging.getLogger(__name__)
@@ -49,6 +49,30 @@ def add_contact(name: str, address: str, step: any, **kwargs) -> dict:
         'next_action': None
     }
 
+def edit_contact(name: str, address: str, step: any, **kwargs) -> dict:
+    
+    redis_client = kwargs.get('redis_client')
+    executor = kwargs.get('executor')
+    providers = kwargs.get('providers')
+    w3 = providers['ethereum']['w3']
+    ens_name = resolve_ens_name(w3, address)
+    code, res = redis_edit_contact(redis_client, executor, name, address, ens_name)
+    return {
+        'result': {'code': code, 'content': res},
+        'action': None,
+        'next_action': None
+    }
+
+def delete_contact(name: str, address: str, step: any, **kwargs) -> dict:
+    redis_client = kwargs.get('redis_client')
+    executor = kwargs.get('executor')
+    code, res = redis_delete_contact(redis_client, executor, name)
+    return {
+        'result': {'code': code, 'content': res},
+        'action': None,
+        'next_action': None
+    }
+
 def show_contact_by_name(name: str, step: any, **kwargs) -> dict:
     """
     Shows contact details by name.
