@@ -68,7 +68,8 @@ def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', ch
     redis_client = kwargs.get("redis_client")
     spender_info = kwargs.get('spender')
     providers = kwargs.get("providers")
-    
+    executor_balance = kwargs.get("executor_balance")
+
     if not providers or chain_raw not in providers:
         raise ValueError(f"Provider for chain {chain} not found")
     
@@ -85,11 +86,11 @@ def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', ch
     print(f"w3: {w3}", flush=True)
 
     if run_mode.lower() == 'auto':
-        return handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address, spender_address, sender_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore)
+        return handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address, spender_address, sender_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore, executor_balance)
     else:
         return handle_manual_transfer(w3, eth_w3, executor, receiver, amount, token_address, redis_client, chain_raw, token_raw)
 
-def handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address, spender_address, sender_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore):
+def handle_auto_transfer(w3: Web3, eth_w3, executor, receiver, amount, token_address, spender_address, sender_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore, executor_balance):
     token_contract = w3.eth.contract(address=token_address, abi=ALLOWANCE_ABI)
     allowance = token_contract.functions.allowance(Web3.to_checksum_address(executor), spender_address).call()
     allowance_normalized = Web3.from_wei(allowance, 'mwei')
@@ -109,17 +110,20 @@ def handle_auto_transfer(w3, eth_w3, executor, receiver, amount, token_address,
     print(f"to: {to}, executor: {executor}")
     
     try:
-        tx_hash = send_transaction(w3, control_contract, sender_address, token_address, executor, to, amount_to_transfer, private_key)
-        tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)
-        status = 200 if tx_receipt['status'] == 1 else 500
-        info = {
-            "content": generate_markdown_content(tx_receipt['status'], tx_hash.hex(), blockchain_explore),
-            "tx_hash": tx_hash.hex(),
-            "chain": chain_raw,
-            "status": tx_receipt['status'],
-            "code": status
-        }
-        return {'result': info, 'action': None, 'next_action': None}
+        can_send, tx_hash = maybe_send_transaction(w3, control_contract, sender_address, token_address, executor, to, amount_to_transfer, private_key, chain_raw, executor_balance)
+        if can_send:
+            tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)
+            status = 200 if tx_receipt['status'] == 1 else 500
+            info = {
+                "content": generate_markdown_content(tx_receipt['status'], tx_hash.hex(), blockchain_explore),
+                "tx_hash": tx_hash.hex(),
+                "chain": chain_raw,
+                "status": tx_receipt['status'],
+                "code": status
+            }
+            return {'result': info, 'action': None, 'next_action': None}
+        else:
+            return {'result': {"code": 500, "content": "Insufficient balance. Estimated gas cost exceed account balance"}, 'action': None, 'next_action': None}
     except Exception as e:
         return handle_transaction_error(e, control_contract, w3)
 
@@ -201,7 +205,7 @@ def handle_incorrect_network(w3, eth_w3, executor, receiver, amount, chain_raw,
 
     return {'result': {'code': 200, 'content': 'Incorrect network, switching to the correct network for you.'}, 'action': action.__dict__, 'next_action': next_action.__dict__}
 
-def send_transaction(w3: Web3, control_contract, sender_address, token, executor, to, amount_to_transfer, private_key):
+def maybe_send_transaction(w3: Web3, control_contract, sender_address, token, executor, to, amount_to_transfer, private_key, chain=None, executor_balance=None):
     nonce = w3.eth.get_transaction_count(sender_address)
     current_gas_price = w3.eth.gas_price
     increase = current_gas_price * 25  // 100
@@ -214,8 +218,14 @@ def send_transaction(w3: Web3, control_contract, sender_address, token, executor
         })
     gas_estimate = w3.eth.estimate_gas(tx)
     tx['gas'] = gas_estimate
+    gas_cost = gas_estimate * new_gas_price
+    
+    print(f"gas_estimate: {gas_estimate}, gas_cost: {gas_cost}, executor_balance: {executor_balance}")
+    # if executor_balance[chain] < gas_cost:
+    #     return False, 0x0
+    
     signed_tx = w3.eth.account.sign_transaction(tx, private_key)
-    return w3.eth.send_raw_transaction(signed_tx.rawTransaction)
+    return True, w3.eth.send_raw_transaction(signed_tx.rawTransaction)
 
 def handle_transaction_error(e, control_contract, w3):
     name, params = decode_custom_error(control_contract.abi, w3, str(e))
